# import files
require "#{File.dirname(__FILE__)}/lrc_reporting"
require "#{File.dirname(__FILE__)}/lrc_resource_reporter"
require "#{File.dirname(__FILE__)}/lrc_uploader"

module ChefHandlerLrc
  module LrcHooks
    attr_reader :lrc_uploader, :lrc_report_handler, :lrc_reporter

    # Provide a chef-client cookbook friendly option
    def lrc_server_url(url)
      lrc_server_options(url: url)
    end

    # {:url => '', ...}
    def lrc_server_options(options = {})
      options[:client_key] = client_key || '/etc/chef/client.pem' unless options[:client_key]
      raise 'No LRC URL! Please provide a URL' unless options[:url]
      @lrc_uploader                 = LrcUploader.new(options)
      @lrc_report_handler.uploader  = @lrc_uploader if @lrc_report_handler
      @lrc_reporter.uploader        = @lrc_uploader if @lrc_reporter
    end

    def lrc_reports_upload(upload, mode = 1)
      return unless upload
      case mode
      when 1
        @lrc_reporter                   = LrcResourceReporter.new(nil)
        @lrc_reporter.uploader          = @lrc_uploader
        @lrc_reporter.log_level         = @lrc_reports_level
        if Chef::Config[:event_handlers].is_a?(Array)
          Chef::Config[:event_handlers].push @lrc_reporter
        else
          Chef::Config[:event_handlers] = [@lrc_reporter]
        end
      when 2
        @lrc_report_handler          = LrcReporting.new
        @lrc_report_handler.uploader = @lrc_uploader
        report_handlers << @lrc_report_handler
        exception_handlers << @lrc_report_handler
      else
        raise ArgumentError, 'unknown mode: ' + mode.to_s
      end
    end

    # level can be string error notice debug
    def reports_log_level(level)
      raise ArgumentError, 'unknown level: ' + level.to_s unless %w(error notice debug).include?(level)
      @lrc_reports_level = level
      return unless @lrc_reporter
      @lrc_reporter.log_level = level
    end
  end
end
